import { NormalizedScalarsMap, ConvertNameFn, BaseVisitorConvertOptions, ParsedEnumValuesMap } from "@graphql-codegen/visitor-plugin-common";
import { VariableNode, TypeNode, ValueNode, NameNode } from "graphql";
import { PLVariableInfo } from "./types";
export interface VariableLike {
    readonly name?: NameNode;
    readonly variable?: VariableNode;
    readonly type: TypeNode;
    readonly defaultValue?: ValueNode;
}
/** Factory to help transform operation variable definitions to
 * target language variables.
 */
export declare class OperationVariablesToObject {
    protected _scalars: NormalizedScalarsMap;
    protected _convertName: ConvertNameFn<BaseVisitorConvertOptions>;
    protected _enumNames: string[];
    protected _enumPrefix: boolean;
    protected _enumValues: ParsedEnumValuesMap;
    constructor(_scalars: NormalizedScalarsMap, _convertName: ConvertNameFn<BaseVisitorConvertOptions>, _enumNames?: string[], _enumPrefix?: boolean, _enumValues?: ParsedEnumValuesMap);
    getName: <T extends VariableLike>(node: T) => string;
    /** Create `<varname>: <typename>` strings foreach operation variable.
     * Still needs scala val/var and default values added.
     */
    transform<T extends VariableLike>(variablesNode: ReadonlyArray<T>): Array<PLVariableInfo>;
    /** None, in ts land, they generate Maybes this way. */
    wrapAstTypeWithModifiers: (baseType: string, typeNode: TypeNode) => string;
    /** In ts world, they output a type mapper for scalars. scala
     * uses the resulting scalar value directly. But in scala3 we could
     * use this feature but it may not be worth it.
     */
    protected getScalar: (name: string) => string;
    protected transformVariable: <T extends VariableLike>(variable: T) => PLVariableInfo;
    protected formatFieldString(fieldName: string, isNonNullType: boolean, isListType: boolean, hasDefaultValue: boolean): string;
    /** Always use js.UndefOr for operation variables.
     * If can be "missing" which is what isNonNullType means here,
     */
    protected formatTypeString(fieldType: string, isNonNullType: boolean, isListType: boolean, hasDefaultValue: boolean): string;
}
export default OperationVariablesToObject;
