"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const visitor_plugin_common_1 = require("@graphql-codegen/visitor-plugin-common");
const graphql_1 = require("graphql");
/** Factory to help transform operation variable definitions to
 * target language variables.
 */
class OperationVariablesToObject {
    constructor(_scalars, _convertName, _enumNames = [], _enumPrefix = true, _enumValues = {}) {
        this._scalars = _scalars;
        this._convertName = _convertName;
        this._enumNames = _enumNames;
        this._enumPrefix = _enumPrefix;
        this._enumValues = _enumValues;
        this.getName = (node) => {
            if (node.name) {
                // who would replace the node.name with just a string? AST borken?
                if (typeof node.name === "string") {
                    return node.name;
                }
                return node.name.value;
            }
            else if (node.variable) {
                return node.variable.name.value;
            }
            return null;
        };
        /** None, in ts land, they generate Maybes this way. */
        this.wrapAstTypeWithModifiers = (baseType, typeNode) => {
            return baseType;
        };
        /** In ts world, they output a type mapper for scalars. scala
         * uses the resulting scalar value directly. But in scala3 we could
         * use this feature but it may not be worth it.
         */
        this.getScalar = (name) => name;
        this.transformVariable = (variable) => {
            let typeValue = null;
            const prefix = "";
            //console.log("transfcormVariable type", variable);
            // if type is a string, use it directly
            if (typeof variable.type === "string") {
                typeValue = variable.type;
            }
            else {
                const baseType = visitor_plugin_common_1.getBaseTypeNode(variable.type);
                const typeName = baseType.name.value;
                if (this._scalars[typeName]) {
                    //typeValue = this.getScalar(typeName);
                    typeValue = this._scalars[typeName];
                }
                else if (this._enumValues[typeName] && this._enumValues[typeName].sourceFile) {
                    typeValue =
                        this._enumValues[typeName].typeIdentifier || this._enumValues[typeName].sourceIdentifier;
                }
                else {
                    // ???
                    typeValue = `${prefix}${this._convertName(baseType, {
                        useTypesPrefix: this._enumNames.includes(typeName) ? this._enumPrefix : true
                    })}`;
                }
            }
            const fieldName = this.getName(variable);
            const fieldType = this.wrapAstTypeWithModifiers(typeValue, variable.type);
            const hasDefaultValue = variable.defaultValue != null && typeof variable.defaultValue !== "undefined";
            const isNonNullType = variable.type.kind === graphql_1.Kind.NON_NULL_TYPE;
            const isListType = variable.type.kind === graphql_1.Kind.LIST_TYPE;
            const formattedFieldString = this.formatFieldString(fieldName, isNonNullType, isListType, hasDefaultValue);
            const formattedTypeString = this.formatTypeString(fieldType, isNonNullType, isListType, hasDefaultValue);
            return {
                name: formattedFieldString,
                type: formattedTypeString,
                wrapper: (type) => `js.UndefOr[${type}]`,
                defaultValue: "js.undefined"
            };
        };
    }
    /** Create `<varname>: <typename>` strings foreach operation variable.
     * Still needs scala val/var and default values added.
     */
    transform(variablesNode) {
        if (!variablesNode || variablesNode.length === 0) {
            return [];
        }
        return variablesNode.map(variable => this.transformVariable(variable));
    }
    formatFieldString(fieldName, isNonNullType, isListType, hasDefaultValue) {
        return fieldName;
    }
    /** Always use js.UndefOr for operation variables.
     * If can be "missing" which is what isNonNullType means here,
     */
    formatTypeString(fieldType, isNonNullType, isListType, hasDefaultValue) {
        return isListType ? `js.Array[${fieldType}]` : fieldType;
    }
}
exports.OperationVariablesToObject = OperationVariablesToObject;
exports.default = OperationVariablesToObject;
//# sourceMappingURL=OperationVariablesToObject.js.map