import { NamingConvention, ConvertOptions, ConvertFn, ScalarsMap, NormalizedScalarsMap, EnumValuesMap, ParsedEnumValuesMap, LoadedFragment } from "@graphql-codegen/visitor-plugin-common";
import { GraphQLSchema } from "graphql";
/** Raw configuration filled in by the @graphql-codgen layer. */
export interface RawConfig {
    /** For declaring documents in scala objects. If missing, documents are not added. */
    gqlImport?: string;
    /** Applies to typeNames and enumValues. Example, lower-case#lowerCase or upper-case#upperCase. Default is pascal-case#pascalCase. */
    namingConvention?: NamingConvention;
    fragmentObjectName?: string;
    scalars?: ScalarsMap;
    /** Does not add __typename to the generated type, unless it was specified. Default is false. */
    skipTypename?: boolean;
    /** Override enum values in the config. */
    enumValues?: EnumValuesMap;
    /** Externally loaded fragments. */
    externalFragments?: LoadedFragment[];
}
/** Final processed configuration taking into account defaults. */
export interface Config {
    schema: GraphQLSchema;
    gqlImport: string | null;
    convertName: ConvertFn<ConvertOptions>;
    fragmentObjectName: string;
    scalars: NormalizedScalarsMap;
    skipTypeName: boolean;
    enumValues: ParsedEnumValuesMap;
    /** All fragments, side-loaded as well as originally in the op docs. */
    fragments: LoadedFragment[];
}
/** Given a list of LoadedFragment objects, return the raw FragmentDefinitionNode AST
 * objects.
 */
export declare function fragmentDefinitions(lf: ReadonlyArray<LoadedFragment>): import("graphql").FragmentDefinitionNode[];
/** Make a Config from a RawConfig. */
export declare function makeConfig(schema: GraphQLSchema, raw: RawConfig): Config;
/** Parse an import config element in the form moduleName#propName */
export declare function parseImport(s: string): {
    moduleName: string;
    propName: string;
};
