"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const visitor_plugin_common_1 = require("@graphql-codegen/visitor-plugin-common");
const scalars_1 = require("./scalars");
const enum_values_1 = require("./enum-values");
/** Given a list of LoadedFragment objects, return the raw FragmentDefinitionNode AST
 * objects.
 */
function fragmentDefinitions(lf) {
    return lf.map(lf => lf.node);
}
exports.fragmentDefinitions = fragmentDefinitions;
/** Make a Config from a RawConfig. */
function makeConfig(schema, raw) {
    const parsedScalarsMap = visitor_plugin_common_1.buildScalars(schema, raw.scalars, scalars_1.DEFAULT_SCALARS);
    const scalars = {};
    Object.keys(parsedScalarsMap).forEach(key => {
        scalars[key] = parsedScalarsMap[key].type;
    });
    return {
        schema,
        gqlImport: raw.gqlImport || null,
        convertName: visitor_plugin_common_1.convertFactory(raw),
        fragmentObjectName: raw.fragmentObjectName || "Fragment",
        scalars,
        skipTypeName: raw.skipTypename || false,
        enumValues: enum_values_1.parseEnumValues(schema, raw.enumValues),
        fragments: raw.externalFragments || []
    };
}
exports.makeConfig = makeConfig;
/** Parse an import config element in the form moduleName#propName */
function parseImport(s) {
    const [mname, pname] = s.split("#");
    return {
        moduleName: mname,
        propName: pname
    };
}
exports.parseImport = parseImport;
//# sourceMappingURL=config.js.map