"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// copy from @graphql-codegen/visitor-plugin-common
const visitor_plugin_common_1 = require("@graphql-codegen/visitor-plugin-common");
const graphql_1 = require("graphql");
const plugin_helpers_1 = require("@graphql-codegen/plugin-helpers");
function parseEnumValues(schema, mapOrStr = {}) {
    const allTypes = schema.getTypeMap();
    const allEnums = Object.keys(allTypes).filter(t => graphql_1.isEnumType(allTypes[t]));
    if (typeof mapOrStr === "object") {
        for (const enumTypeName of allEnums) {
            const enumType = schema.getType(enumTypeName);
            for (const { name, value } of enumType.getValues()) {
                if (value && value !== name) {
                    mapOrStr[enumTypeName] = mapOrStr[enumTypeName] || {};
                    if (typeof mapOrStr[enumTypeName] !== "string" &&
                        !mapOrStr[enumTypeName][name]) {
                        mapOrStr[enumTypeName][name] = value;
                    }
                }
            }
        }
        const invalidMappings = Object.keys(mapOrStr).filter(gqlName => !allEnums.includes(gqlName));
        if (invalidMappings.length > 0) {
            throw new plugin_helpers_1.DetailedError(`Invalid 'enumValues' mapping!`, `The following types does not exist in your GraphQL schema: ${invalidMappings.join(", ")}`);
        }
        return Object.keys(mapOrStr).reduce((prev, gqlIdentifier) => {
            const pointer = mapOrStr[gqlIdentifier];
            if (typeof pointer === "string") {
                const mapper = visitor_plugin_common_1.parseMapper(pointer, gqlIdentifier);
                return Object.assign(Object.assign({}, prev), { [gqlIdentifier]: {
                        typeIdentifier: gqlIdentifier,
                        sourceFile: mapper.isExternal ? mapper.source : undefined,
                        sourceIdentifier: visitor_plugin_common_1.isExternalMapperType(mapper)
                            ? mapper.import
                            : mapper.type,
                        mappedValues: null
                    } });
            }
            else if (typeof pointer === "object") {
                return Object.assign(Object.assign({}, prev), { [gqlIdentifier]: {
                        typeIdentifier: gqlIdentifier,
                        sourceFile: null,
                        sourceIdentifier: null,
                        mappedValues: pointer
                    } });
            }
            else {
                throw new plugin_helpers_1.DetailedError(`Invalid "enumValues" configuration`, `Enum "${gqlIdentifier}": expected string or object (with enum values mapping)`);
            }
        }, {});
    }
    else if (typeof mapOrStr === "string") {
        return allEnums
            .filter(enumName => !enumName.startsWith("__"))
            .reduce((prev, enumName) => {
            return Object.assign(Object.assign({}, prev), { [enumName]: {
                    typeIdentifier: enumName,
                    sourceFile: mapOrStr,
                    sourceIdentifier: enumName,
                    mappedValues: null
                } });
        }, {});
    }
    return {};
}
exports.parseEnumValues = parseEnumValues;
//# sourceMappingURL=enum-values.js.map