import { GraphQLSchema, GraphQLEnumType } from "graphql";
/** Generate string representing enums. */
export declare function genEnums(schema: GraphQLSchema, options?: EnumGenerationOptions): string;
/** Find all schema entries that are enums. */
export declare function findAllEnums(schema: GraphQLSchema): GraphQLEnumType[];
/** Code generation options for enums. */
export interface EnumGenerationOptions {
    /** Use names as values instead of any values found in the schema. */
    useNames?: boolean;
    /** Upcase the generated scala artifact name. Does not apply to individual enum values. */
    upcase?: boolean;
    /** Exclude some names from being generated. */
    excludes?: Array<string>;
}
/** Default for enum code generation. */
export declare const defaultEnumGenerationOptions: EnumGenerationOptions;
/** Enums are mostly rendered on their own at the top level since
 * they do not have any real dependencies. Generate enum using
 * standard scala.js idiom for enums.
 */
export declare function renderEnumDefinition(t: GraphQLEnumType, options?: EnumGenerationOptions): string;
