"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_1 = require("graphql");
const pascal_case_1 = require("pascal-case");
/** Generate string representing enums. */
function genEnums(schema, options) {
    options = Object.assign(Object.assign({}, exports.defaultEnumGenerationOptions), options);
    return findAllEnums(schema)
        .map(t => options.excludes.includes(t.name)
        ? null
        : renderEnumDefinition(t, options))
        .filter(a => a)
        .join("\n");
}
exports.genEnums = genEnums;
/** Find all schema entries that are enums. */
function findAllEnums(schema) {
    return Object.entries(schema.getTypeMap())
        .map(kv => {
        const n = kv[0];
        const t = kv[1];
        return graphql_1.isEnumType(t) ? t : null;
    })
        .filter(a => a);
}
exports.findAllEnums = findAllEnums;
/** Default for enum code generation. */
exports.defaultEnumGenerationOptions = {
    useNames: false,
    upcase: true,
    excludes: ["__DirectiveLocation", "__TypeKind", "CacheControlScope"]
};
/** Enums are mostly rendered on their own at the top level since
 * they do not have any real dependencies. Generate enum using
 * standard scala.js idiom for enums.
 */
function renderEnumDefinition(t, options) {
    var _a;
    options = Object.assign(Object.assign({}, exports.defaultEnumGenerationOptions), options);
    let str = [];
    const name = options.upcase ? pascal_case_1.pascalCase(t.name) : t.name;
    str.push(`/** ${(_a = t.description) !== null && _a !== void 0 ? _a : "Enum " + name}`);
    str.push(` * Schema name: ${t.name}`);
    str.push(" */");
    str.push("@js.native");
    str.push(`abstract trait ${name} extends js.Any`);
    str.push(`object ${name} {`);
    // for each value generate a val
    t.getValues().forEach(v => {
        const vname = v.name;
        const vvalue = options.useNames
            ? vname
            : v.value
                ? v.value.toString()
                : vname;
        str.push(`val ${vname} = "${vvalue}".asInstanceOf[${name}]`);
    });
    str.push("}");
    return str.join("\n");
}
exports.renderEnumDefinition = renderEnumDefinition;
//# sourceMappingURL=enums.js.map