"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const dependency_graph_1 = require("dependency-graph");
const graphql_1 = require("graphql");
/** Find all FragmentDefinitionNode children. Dedup circles using a dependency graph. */
function getFragments(fragments) {
    const graph = fragmentsGraph(fragments);
    const orderedDeps = graph.overallOrder();
    const localFragments = orderedDeps.filter(name => !graph.getNodeData(name).isExternal);
    return localFragments.map(name => graph.getNodeData(name).node);
}
exports.getFragments = getFragments;
/** Generate graph of names=>document content mappings. */
function fragmentsGraph(fragments) {
    const graph = new dependency_graph_1.DepGraph({ circular: true });
    for (const fragment of fragments) {
        if (graph.hasNode(fragment.name)) {
            const cachedAsString = graphql_1.print(graph.getNodeData(fragment.name).node);
            const asString = graphql_1.print(fragment.node);
            if (cachedAsString !== asString) {
                throw new Error(`Duplicated fragment called '${fragment.name}'!`);
            }
        }
        graph.addNode(fragment.name, fragment);
    }
    fragments.forEach(fragment => {
        const depends = extractFragments(fragment.node);
        if (depends && depends.length > 0) {
            depends.forEach(name => {
                graph.addDependency(fragment.name, name);
            });
        }
    });
    return graph;
}
exports.fragmentsGraph = fragmentsGraph;
/** Extract out all FragmentSpread names in children AST nodes. */
function extractFragments(document) {
    if (!document) {
        return [];
    }
    const names = [];
    graphql_1.visit(document, {
        enter: {
            FragmentSpread: (node) => {
                names.push(node.name.value);
            }
        }
    });
    return names;
}
exports.extractFragments = extractFragments;
//# sourceMappingURL=fragments.js.map