"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
// https://graphql-code-generator.com/docs/custom-codegen/write-your-plugin
require("source-map-support/register");
const graphql_1 = require("graphql");
const operations_1 = require("./operations");
const config_1 = require("./config");
const enums_1 = require("./enums");
exports.plugin = (schema, documents, config) => {
    const allAst = graphql_1.concatAST(documents.map(v => v.document));
    // Gather all fragments into one place since finding them on the
    // fly in the schema is painful.
    const allFragments = [
        ...allAst.definitions.filter(d => d.kind === graphql_1.Kind.FRAGMENT_DEFINITION).map(fragmentDef => ({
            node: fragmentDef,
            name: fragmentDef.name.value,
            onType: fragmentDef.typeCondition.name.value,
            isExternal: false
        })),
        ...(config.externalFragments || [])
    ];
    // console.log("all types");
    // Object.keys(schema.getTypeMap())
    //   .sort()
    //   .forEach(k => {
    //     const t = schema.getType(k);
    //     console.log("type: ", k, ": ", t);
    //   });
    try {
        const final_config = config_1.makeConfig(schema, Object.assign(Object.assign({}, config), { externalFragments: allFragments }));
        //const ops = extractOperations(allAst).map(
        const visitor = new operations_1.ScalaJSVisitor(final_config);
        const visitorResult = graphql_1.visit(allAst, { leave: visitor });
        return {
            prepend: [
                ...operations_1.genImports(["scala.scalajs.js", "js.|", final_config.gqlImport]),
                visitor.op_comment
            ],
            content: [
                // `
                // object __Conversions {
                //   implicit class RichOrNull[T](val v: T |Null) {
                //     def forceGet = if(v == null) null else v.asInstanceOf[T]
                //   }
                // }
                // import __Conversions._
                // `,
                enums_1.genEnums(schema),
                operations_1.getFragmentBlock(final_config.fragments, final_config.fragmentObjectName, final_config.convertName, node => operations_1.makeGQLForScala(node, final_config.fragmentObjectName, final_config.convertName)),
                ...visitorResult.definitions.filter(t => typeof t === "string")
            ].join("\n")
        };
    }
    catch (e) {
        console.log("Error generating content", e);
        return { content: `Error generating content ${e.message}` };
    }
};
exports.validate = (schema, documents, config, outputFile) => __awaiter(void 0, void 0, void 0, function* () {
    if (!outputFile.endsWith(".scala")) {
        throw new Error(`Plugin "scalajs-client" requires output file extension to be ".scala"!`);
    }
});
//# sourceMappingURL=index.js.map