import { OperationTypeNode, GraphQLSchema, GraphQLType } from "graphql";
/** Programming language variable broken out so it can be
 * used in multiple contexts.
 */
export interface PLVariableInfo {
    /** Varible name. */
    readonly name: string;
    /** Base type without wrapping for null/optionality. May have
     * list wrapper though so it may not be the element type itself.
     */
    readonly type: string;
    /** Call the wrapper to generate the full type signature. */
    readonly wrapper: (type: string) => string;
    /** Default value, raw string. scala may need converters here. */
    readonly defaultValue?: string;
}
/** Given an OperationTypeNode , return the root (query,mutation,subscription) in the schema. */
export declare function getRootType(operation: OperationTypeNode, schema: GraphQLSchema): import("graphql").GraphQLObjectType<any, any, {
    [key: string]: any;
}>;
/*** Debug print a type using recursion to unwrap if needed. */
export declare function debug_type(t: GraphQLType, indent?: number, log?: (message?: any, ...optionalParams: any[]) => void): void;
export interface WrapperOptions {
    mkList: (arg: string) => string;
    mkOpt: (arg: string) => string;
    mkOptZero: () => string;
    mkListZero: () => string;
}
export declare const defaultWrapperOptions: WrapperOptions;
/** Build a rendering thunk for the type. Takes
 * into account null and list types only i.e. wrapping types.
 */
export declare function mk_type_wrapper_thunk(t: GraphQLType, options?: Partial<WrapperOptions>): (arg: string) => string;
